@@ -18,11 +18,8 @@
 #endif
 
 #include "postgres_fe.h"
-
 #include "libpq-fe.h"
-#include "pqexpbuffer.h"
 
-#include <fcntl.h>
 #include <locale.h>
 #include <signal.h>
 #include <sys/types.h>
@@ -297,84 +294,50 @@ get_pgpid(void)
 static char **
 readfile(const char *path)
 {
-	int			fd;
-	int			nlines;
+	FILE	   *infile;
+	int			maxlength = 1,
+				linelen = 0;
+	int			nlines = 0;
 	char	  **result;
 	char	   *buffer;
-	char	   *linebegin;
-	int			i;
-	int			n;
-	int			len;
-	struct stat	statbuf;
+	int			c;
 
-	/*
-	 * Slurp the file into memory.
-	 *
-	 * The file can change concurrently, so we read the whole file into memory
-	 * with a single read() call. That's not guaranteed to get an atomic
-	 * snapshot, but in practice, for a small file, it's close enough for the
-	 * current use.
-	 */
-	fd = open(path, O_RDONLY | PG_BINARY, 0);
-	if (fd < 0)
+	if ((infile = fopen(path, "r")) == NULL)
 		return NULL;
-	if (fstat(fd, &statbuf) < 0)
-	{
-		close(fd);
-		return NULL;
-	}
-	if (statbuf.st_size == 0)
-	{
-		/* empty file */
-		close(fd);
-		result = (char **) pg_malloc(sizeof(char *));
-		*result = NULL;
-		return result;
-	}
-	buffer = pg_malloc(statbuf.st_size + 1);
 
-	len = read(fd, buffer, statbuf.st_size + 1);
-	close(fd);
-	if (len != statbuf.st_size)
-	{
-		/* oops, the file size changed between fstat and read */
-		free(buffer);
-		return NULL;
-	}
+	/* pass over the file twice - the first time to size the result */
 
-	/*
-	 * Count newlines. We expect there to be a newline after each full line,
-	 * including one at the end of file. If there isn't a newline at the end,
-	 * any characters after the last newline will be ignored.
-	 */
-	nlines = 0;
-	for (i = 0; i < len; i++)
+	while ((c = fgetc(infile)) != EOF)
 	{
-		if (buffer[i] == '\n')
-			nlines++;
-	}
-
-	/* set up the result buffer */
-	result = (char **) pg_malloc((nlines + 1) * sizeof(char *));
-
-	/* now split the buffer into lines */
-	linebegin = buffer;
-	n = 0;
-	for (i = 0; i < len; i++)
-	{
-		if (buffer[i] == '\n')
+		linelen++;
+		if (c == '\n')
 		{
-			int		slen = &buffer[i] - linebegin + 1;
-			char   *linebuf = pg_malloc(slen + 1);
-			memcpy(linebuf, linebegin, slen);
-			linebuf[slen] = '\0';
-			result[n++] = linebuf;
-			linebegin = &buffer[i + 1];
+			nlines++;
+			if (linelen > maxlength)
+				maxlength = linelen;
+			linelen = 0;
 		}
 	}
-	result[n] = NULL;
 
+	/* handle last line without a terminating newline (yuck) */
+	if (linelen)
+		nlines++;
+	if (linelen > maxlength)
+		maxlength = linelen;
+
+	/* set up the result and the line buffer */
+	result = (char **) pg_malloc((nlines + 1) * sizeof(char *));
+	buffer = (char *) pg_malloc(maxlength + 1);
+
+	/* now reprocess the file and store the lines */
+	rewind(infile);
+	nlines = 0;
+	while (fgets(buffer, maxlength + 1, infile) != NULL)
+		result[nlines++] = xstrdup(buffer);
+
+	fclose(infile);
 	free(buffer);
+	result[nlines] = NULL;
 
 	return result;
 }
@@ -1040,13 +1003,16 @@ pgwin32_IsInstalled(SC_HANDLE hSCM)
 static char *
 pgwin32_CommandLine(bool registration)
 {
-	PQExpBuffer cmdLine = createPQExpBuffer();
-	char		cmdPath[MAXPGPATH];
+	static char cmdLine[MAXPGPATH];
 	int			ret;
 
+#ifdef __CYGWIN__
+	char		buf[MAXPGPATH];
+#endif
+
 	if (registration)
 	{
-		ret = find_my_exec(argv0, cmdPath);
+		ret = find_my_exec(argv0, cmdLine);
 		if (ret != 0)
 		{
 			write_stderr(_("%s: could not find own program executable\n"), progname);
@@ -1056,7 +1022,7 @@ pgwin32_CommandLine(bool registration)
 	else
 	{
 		ret = find_other_exec(argv0, "postgres", PG_BACKEND_VERSIONSTR,
-							  cmdPath);
+							  cmdLine);
 		if (ret != 0)
 		{
 			write_stderr(_("%s: could not find postgres program executable\n"), progname);
@@ -1066,57 +1032,50 @@ pgwin32_CommandLine(bool registration)
 
 #ifdef __CYGWIN__
 	/* need to convert to windows path */
-	{
-		char		buf[MAXPGPATH];
-
-#if CYGWIN_VERSION_DLL_MAJOR >= 1007
-		cygwin_conv_path(CCP_POSIX_TO_WIN_A, cmdPath, buf, sizeof(buf));
-#else
-		cygwin_conv_to_full_win32_path(cmdPath, buf);
+	cygwin_conv_to_full_win32_path(cmdLine, buf);
+	strcpy(cmdLine, buf);
 #endif
-		strcpy(cmdPath, buf);
-	}
-#endif
-
-	/* if path does not end in .exe, append it */
-	if (strlen(cmdPath) < 4 ||
-		pg_strcasecmp(cmdPath + strlen(cmdPath) - 4, ".exe") != 0)
-		snprintf(cmdPath + strlen(cmdPath), sizeof(cmdPath) - strlen(cmdPath),
-				 ".exe");
-
-	/* use backslashes in path to avoid problems with some third-party tools */
-	make_native_path(cmdPath);
-
-	/* be sure to double-quote the executable's name in the command */
-	appendPQExpBuffer(cmdLine, "\"%s\"", cmdPath);
-
-	/* append assorted switches to the command line, as needed */
 
 	if (registration)
-		appendPQExpBuffer(cmdLine, " runservice -N \"%s\"",
-						  register_servicename);
+	{
+		if (pg_strcasecmp(cmdLine + strlen(cmdLine) - 4, ".exe"))
+		{
+			/* If commandline does not end in .exe, append it */
+			strcat(cmdLine, ".exe");
+		}
+		strcat(cmdLine, " runservice -N \"");
+		strcat(cmdLine, register_servicename);
+		strcat(cmdLine, "\"");
+	}
 
 	if (pg_data)
-		appendPQExpBuffer(cmdLine, " -D \"%s\"", pg_data);
+	{
+		strcat(cmdLine, " -D \"");
+		strcat(cmdLine, pg_data);
+		strcat(cmdLine, "\"");
+	}
 
 	if (registration && do_wait)
-		appendPQExpBuffer(cmdLine, " -w");
+		strcat(cmdLine, " -w");
 
 	if (registration && wait_seconds != DEFAULT_WAIT)
-		appendPQExpBuffer(cmdLine, " -t %d", wait_seconds);
+		/* concatenate */
+		sprintf(cmdLine + strlen(cmdLine), " -t %d", wait_seconds);
 
 	if (registration && silent_mode)
-		appendPQExpBuffer(cmdLine, " -s");
+		strcat(cmdLine, " -s");
 
 	if (post_opts)
 	{
+		strcat(cmdLine, " ");
 		if (registration)
-			appendPQExpBuffer(cmdLine, " -o \"%s\"", post_opts);
-		else
-			appendPQExpBuffer(cmdLine, " %s", post_opts);
+			strcat(cmdLine, " -o \"");
+		strcat(cmdLine, post_opts);
+		if (registration)
+			strcat(cmdLine, "\"");
 	}
 
-	return cmdLine->data;
+	return cmdLine;
 }
 
 static void
@@ -1552,7 +1511,7 @@ CreateRestrictedProcess(char *cmd, PROCESS_INFORMATION *processInfo, bool as_ser
 	 */
 	return r;
 }
-#endif   /* defined(WIN32) || defined(__CYGWIN__) */
+#endif
 
 static void
 do_advice(void)
