@@ -116,7 +116,7 @@ static int ldapServiceLookup(const char *purl, PQconninfoOption *options,
  *
  * PQconninfoOptions[] is a constant static array that we use to initialize
  * a dynamically allocated working copy.  All the "val" fields in
- * PQconninfoOptions[] *must* be NULL.  In a working copy, non-null "val"
+ * PQconninfoOptions[] *must* be NULL.	In a working copy, non-null "val"
  * fields point to malloc'd strings that should be freed when the working
  * array is freed (see PQconninfoFree).
  * ----------
@@ -181,7 +181,7 @@ static const PQconninfoOption PQconninfoOptions[] = {
 	 * to exclude them since none of them are mandatory.
 	 */
 	{"sslmode", "PGSSLMODE", DefaultSSLMode, NULL,
-	"SSL-Mode", "", 12},			/* sizeof("verify-full") == 12 */
+	"SSL-Mode", "", 8},			/* sizeof("disable") == 8 */
 
 	{"sslcert", "PGSSLCERT", NULL, NULL,
 	"SSL-Client-Cert", "", 64},
@@ -324,7 +324,7 @@ PQconnectdb(const char *conninfo)
  * See comment for PQconnectdb for the definition of the string format.
  *
  * Returns a PGconn*.  If NULL is returned, a malloc error has occurred, and
- * you should not attempt to proceed with this connection.  If the status
+ * you should not attempt to proceed with this connection.	If the status
  * field of the connection returned is CONNECTION_BAD, an error has
  * occurred. In this case you should call PQfinish on the result, (perhaps
  * inspecting the error message first).  Other fields of the structure may not
@@ -566,14 +566,14 @@ connectOptions2(PGconn *conn)
  *
  * Parse an empty string like PQconnectdb() would do and return the
  * resulting connection options array, ie, all the default values that are
- * available from the environment etc.  On error (eg out of memory),
+ * available from the environment etc.	On error (eg out of memory),
  * NULL is returned.
  *
  * Using this function, an application may determine all possible options
  * and their current default values.
  *
  * NOTE: as of PostgreSQL 7.0, the returned array is dynamically allocated
- * and should be freed when no longer needed via PQconninfoFree().  (In prior
+ * and should be freed when no longer needed via PQconninfoFree().	(In prior
  * versions, the returned array was static, but that's not thread-safe.)
  * Pre-7.0 applications that use this function will see a small memory leak
  * until they are updated to call PQconninfoFree.
@@ -790,7 +790,7 @@ static int
 connectDBStart(PGconn *conn)
 {
 	int			portnum;
-	char		portstr[MAXPGPATH];
+	char		portstr[128];
 	struct addrinfo *addrs = NULL;
 	struct addrinfo hint;
 	const char *node;
@@ -842,15 +842,6 @@ connectDBStart(PGconn *conn)
 		node = NULL;
 		hint.ai_family = AF_UNIX;
 		UNIXSOCK_PATH(portstr, portnum, conn->pgunixsocket);
-		if (strlen(portstr) >= UNIXSOCK_PATH_BUFLEN)
-		{
-			appendPQExpBuffer(&conn->errorMessage,
-							  libpq_gettext("Unix-domain socket path \"%s\" is too long (maximum %d bytes)\n"),
-											portstr,
-											(int) (UNIXSOCK_PATH_BUFLEN - 1));
-			conn->options_valid = false;
-			goto connect_errReturn;
-		}
 #else
 		/* Without Unix sockets, default to localhost instead */
 		node = "localhost";
@@ -952,7 +943,7 @@ connectDBComplete(PGconn *conn)
 	for (;;)
 	{
 		/*
-		 * Wait, if necessary.  Note that the initial state (just after
+		 * Wait, if necessary.	Note that the initial state (just after
 		 * PQconnectStart) is to wait for the socket to select for writing.
 		 */
 		switch (flag)
@@ -1014,7 +1005,7 @@ connectDBComplete(PGconn *conn)
  *		will not block.
  *	 o	If you do not supply an IP address for the remote host (i.e. you
  *		supply a host name instead) then PQconnectStart will block on
- *		gethostbyname.  You will be fine if using Unix sockets (i.e. by
+ *		gethostbyname.	You will be fine if using Unix sockets (i.e. by
  *		supplying neither a host name nor a host address).
  *	 o	If your backend wants to use Kerberos authentication then you must
  *		supply both a host name and a host address, otherwise this function
@@ -1170,10 +1161,9 @@ keep_going:						/* We will come back to here until there is
 								addr_cur->ai_addrlen) < 0)
 					{
 						if (SOCK_ERRNO == EINPROGRESS ||
-#ifdef WIN32
 							SOCK_ERRNO == EWOULDBLOCK ||
-#endif
-							SOCK_ERRNO == EINTR)
+							SOCK_ERRNO == EINTR ||
+							SOCK_ERRNO == 0)
 						{
 							/*
 							 * This is fine - we're in non-blocking mode, and
@@ -1199,7 +1189,7 @@ keep_going:						/* We will come back to here until there is
 					/*
 					 * This connection failed --- set up error report, then
 					 * close socket (do it this way in case close() affects
-					 * the value of errno...).  We will ignore the connect()
+					 * the value of errno...).	We will ignore the connect()
 					 * failure and keep going if there are more addresses.
 					 */
 					connectFailureMessage(conn, SOCK_ERRNO);
@@ -2094,7 +2084,7 @@ freePGconn(PGconn *conn)
  *	 - properly close a connection to the backend
  *
  * This should reset or release all transient state, but NOT the connection
- * parameters.  On exit, the PGconn should be in condition to start a fresh
+ * parameters.	On exit, the PGconn should be in condition to start a fresh
  * connection with the same parameters (see PQreset()).
  */
 static void
@@ -2230,7 +2220,7 @@ PQreset(PGconn *conn)
 		if (connectDBStart(conn) && connectDBComplete(conn))
 		{
 			/*
-			 * Notify event procs of successful reset.  We treat an event proc
+			 * Notify event procs of successful reset.	We treat an event proc
 			 * failure as disabling the connection ... good idea?
 			 */
 			int			i;
@@ -2290,7 +2280,7 @@ PQresetPoll(PGconn *conn)
 		if (status == PGRES_POLLING_OK)
 		{
 			/*
-			 * Notify event procs of successful reset.  We treat an event proc
+			 * Notify event procs of successful reset.	We treat an event proc
 			 * failure as disabling the connection ... good idea?
 			 */
 			int			i;
@@ -2479,7 +2469,7 @@ cancel_errReturn:
  * Returns TRUE if able to send the cancel request, FALSE if not.
  *
  * On failure, an error message is stored in *errbuf, which must be of size
- * errbufsize (recommended size is 256 bytes).  *errbuf is not changed on
+ * errbufsize (recommended size is 256 bytes).	*errbuf is not changed on
  * success return.
  */
 int
@@ -2610,13 +2600,11 @@ ldapServiceLookup(const char *purl, PQconninfoOption *options,
 	int			port = LDAP_DEF_PORT,
 				scope,
 				rc,
+				msgid,
 				size,
 				state,
 				oldstate,
 				i;
-#ifndef WIN32
-	int			msgid;
-#endif
 	bool		found_keyword;
 	char	   *url,
 			   *hostname,
@@ -2760,39 +2748,12 @@ ldapServiceLookup(const char *purl, PQconninfoOption *options,
 	}
 
 	/*
-	 * Perform an explicit anonymous bind.
-	 *
-	 * LDAP does not require that an anonymous bind is performed explicitly,
-	 * but we want to distinguish between the case where LDAP bind does not
-	 * succeed within PGLDAP_TIMEOUT seconds (return 2 to continue parsing
-	 * the service control file) and the case where querying the LDAP server
-	 * fails (return 1 to end parsing).
-	 *
-	 * Unfortunately there is no way of setting a timeout that works for
-	 * both Windows and OpenLDAP.
+	 * Initialize connection to the server.  We do an explicit bind because we
+	 * want to return 2 if the bind fails.
 	 */
-#ifdef WIN32
-	/* the nonstandard ldap_connect function performs an anonymous bind */
-	if (ldap_connect(ld, &time) != LDAP_SUCCESS)
-	{
-		/* error or timeout in ldap_connect */
-		free(url);
-		ldap_unbind(ld);
-		return 2;
-	}
-#else /* !WIN32 */
-	/* in OpenLDAP, use the LDAP_OPT_NETWORK_TIMEOUT option */
-	if (ldap_set_option(ld, LDAP_OPT_NETWORK_TIMEOUT, &time) != LDAP_SUCCESS)
-	{
-		free(url);
-		ldap_unbind(ld);
-		return 3;
-	}
-
-	/* anonymous bind */
 	if ((msgid = ldap_simple_bind(ld, NULL, NULL)) == -1)
 	{
-		/* error or network timeout */
+		/* error in ldap_simple_bind() */
 		free(url);
 		ldap_unbind(ld);
 		return 2;
@@ -2803,25 +2764,18 @@ ldapServiceLookup(const char *purl, PQconninfoOption *options,
 	if ((rc = ldap_result(ld, msgid, LDAP_MSG_ALL, &time, &res)) == -1 ||
 		res == NULL)
 	{
-		/* error or timeout */
 		if (res != NULL)
+		{
+			/* timeout */
 			ldap_msgfree(res);
+		}
+		/* error in ldap_result() */
 		free(url);
 		ldap_unbind(ld);
 		return 2;
 	}
 	ldap_msgfree(res);
 
-	/* reset timeout */
-	time.tv_sec = -1;
-	if (ldap_set_option(ld, LDAP_OPT_NETWORK_TIMEOUT, &time) != LDAP_SUCCESS)
-	{
-		free(url);
-		ldap_unbind(ld);
-		return 3;
-	}
-#endif /* WIN32 */
-
 	/* search */
 	res = NULL;
 	if ((rc = ldap_search_st(ld, dn, scope, filter, attrs, 0, &time, &res))
@@ -3035,8 +2989,7 @@ ldapServiceLookup(const char *purl, PQconninfoOption *options,
 
 	return 0;
 }
-
-#endif	/* USE_LDAP */
+#endif
 
 #define MAXBUFSIZE 256
 
@@ -3206,7 +3159,7 @@ parseServiceInfo(PQconninfoOption *options, PQExpBuffer errorMessage)
  *		PQconninfoParse
  *
  * Parse a string like PQconnectdb() would do and return the
- * resulting connection options array.  NULL is returned on failure.
+ * resulting connection options array.	NULL is returned on failure.
  * The result contains only options specified directly in the string,
  * not any possible default values.
  *
@@ -3557,16 +3510,7 @@ PQhost(const PGconn *conn)
 {
 	if (!conn)
 		return NULL;
-	if (conn->pghost != NULL && conn->pghost[0] != '\0')
-		return conn->pghost;
-	else
-	{
-#ifdef HAVE_UNIX_SOCKETS
-		return conn->pgunixsocket;
-#else
-		return DefaultHost;
-#endif
-	}
+	return conn->pghost ? conn->pghost : conn->pgunixsocket;
 }
 
 char *
