@@ -1081,13 +1081,6 @@ select * from another;
 
 drop table another;
 
--- ALTER TYPE with a check constraint and a child table (bug before Nov 2012)
-CREATE TABLE test_inh_check (a float check (a > 10.2));
-CREATE TABLE test_inh_check_child() INHERITS(test_inh_check);
-ALTER TABLE test_inh_check ALTER COLUMN a TYPE numeric;
-\d test_inh_check
-\d test_inh_check_child
-
 -- disallow recursive containment of row types
 create temp table recur1 (f1 int);
 alter table recur1 add column f2 recur1; -- fails
